@@ -24,32 +24,27 @@
  *    Michael Meeks <michael@ximian.com>
  *    Morten Welinder <terra@diku.dk>
  *
- *  Copyright (C) 2000-2001 Ximian Inc. and authors
- *
+ *  Copyright (C) 2000-2003 Ximian Inc. and authors
  */
 
-#define __GNOME_FONT_FACE_C__
-
-#include "config.h"
+#include <config.h>
 #include <stdlib.h>
 #include <ctype.h>
 #include <string.h>
-#include <sys/types.h>
-#include <sys/stat.h>
-#include <sys/mman.h>
-#include <unistd.h>
-#include <fcntl.h>
 #include <stdarg.h>
 #include <locale.h>
-#include <libgnomeprint/gnome-font-private.h>
-#include "gnome-print-i18n.h"
-#include "gnome-fontmap.h"
-#include "gp-truetype-utils.h"
-#include "gp-tt-t1.h"
-#include "gnome-font-family.h"
 
 #include <freetype/ftoutln.h>
 
+#include <libgnomeprint/gnome-print-private.h>
+#include <libgnomeprint/gnome-font-private.h>
+#include <libgnomeprint/gnome-print-i18n.h>
+#include <libgnomeprint/gnome-fontmap.h>
+#include <libgnomeprint/gp-truetype-utils.h>
+#include <libgnomeprint/gp-tt-t1.h>
+#include <libgnomeprint/gnome-font-family.h>
+#include <libgnomeprint/gnome-print-encode.h>
+
 enum {
 	PROP_0,
 	PROP_FONTNAME,
@@ -76,12 +71,12 @@ static void gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GPara
 static void gff_load_metrics (GnomeFontFace *face, gint glyph);
 static void gff_load_outline (GnomeFontFace *face, gint glyph);
 
+static void gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size);
 static void gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...);
 static void gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject * pso, gint size);
 
-#define GFE_IS_T1_A(e) (((e)->type == GP_FONT_ENTRY_TYPE1) || ((e)->type == GP_FONT_ENTRY_TYPE1_ALIAS))
+#define GFE_IS_T1(e) ((e)->type == GP_FONT_ENTRY_TYPE1)
 #define GFE_IS_TT(e) ((e)->type == GP_FONT_ENTRY_TRUETYPE)
-#define GFE_IS_S(e) ((e)->type == GP_FONT_ENTRY_SPECIAL)
 
 static GObjectClass * parent_class;
 
@@ -186,6 +181,11 @@ gnome_font_face_finalize (GObject * object)
 		face->ft_face = NULL;
 	}
 
+	if (face->psname) {
+		g_free (face->psname);
+		face->psname = NULL;
+	}
+
 	/* Fonts should reference us */
 	g_assert (face->fonts == NULL);
 
@@ -203,7 +203,7 @@ gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec
 
 	switch (id) {
 	case PROP_FONTNAME:
-		g_value_set_string (value, face->entry->psname);
+		g_value_set_string (value, face->psname);
 		break;
 	case PROP_FULLNAME:
 		g_value_set_string (value, face->entry->name);
@@ -215,11 +215,11 @@ gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec
 		g_value_set_string (value, face->entry->weight);
 		break;
 	case PROP_ITALICANGLE:
-		/* fixme: implement */
+		/* FIXME: implement (Lauris) */
 		g_value_set_double (value, gnome_font_face_is_italic (face) ? -20.0 : 0.0);
 		break;
 	case PROP_ISFIXEDPITCH:
-		/* fixme: implement */
+		/* FIXME: implement (Lauris) */
 		g_value_set_boolean (value, FALSE);
 		break;
 	case PROP_FONTBBOX:
@@ -239,11 +239,11 @@ gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec
 		g_value_set_string (value, "0.0");
 		break;
 	case PROP_CAPHEIGHT:
-		/* fixme: implement */
+		/* FIXME: implement (Lauris) */
 		g_value_set_double (value, 900.0);
 		break;
 	case PROP_XHEIGHT:
-		/* fixme: implement */
+		/* FIXME: implement (Lauris) */
 		g_value_set_double (value, 600.0);
 		break;
 	case PROP_ASCENDER:
@@ -266,6 +266,16 @@ gnome_font_face_get_prop (GObject *o, guint id, GValue *value, GParamSpec *pspec
  * In future those names can be possibly localized (except ps_name)
  */
 
+
+
+/**
+ * gnome_font_face_get_name:
+ * @face: 
+ * 
+ * Return the name of the Font
+ * 
+ * Return Value: a const pointer to the name, NULL on error
+ **/
 const guchar *
 gnome_font_face_get_name (const GnomeFontFace * face)
 {
@@ -275,6 +285,14 @@ gnome_font_face_get_name (const GnomeFontFace * face)
 	return face->entry->name;
 }
 
+/**
+ * gnome_font_face_get_family_name:
+ * @face: 
+ * 
+ * Get the family name
+ * 
+ * Return Value: a const pointer to the family name, NULL on error
+ **/
 const guchar *
 gnome_font_face_get_family_name (const GnomeFontFace * face)
 {
@@ -284,6 +302,14 @@ gnome_font_face_get_family_name (const GnomeFontFace * face)
 	return face->entry->familyname;
 }
 
+/**
+ * gnome_font_face_get_species_name:
+ * @face: 
+ * 
+ * Get the species name of the font
+ * 
+ * Return Value: a const pointer to the species name, NULL on error
+ **/
 const guchar *
 gnome_font_face_get_species_name (const GnomeFontFace * face)
 {
@@ -293,19 +319,32 @@ gnome_font_face_get_species_name (const GnomeFontFace * face)
 	return face->entry->speciesname;
 }
 
+/**
+ * gnome_font_face_get_ps_name:
+ * @face: 
+ * 
+ * The postscript name of the font. This is the name with which
+ * the font is embeded inside Postscript/PDF jobs.
+ * 
+ * Return Value: a const pointer to the name, NULL on error
+ **/
 const guchar *
 gnome_font_face_get_ps_name (const GnomeFontFace * face)
 {
 	g_return_val_if_fail (face != NULL, NULL);
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), NULL);
 
-	return face->entry->psname;
+	return face->psname;
 }
 
-/*
- * Returns number of defined glyphs in typeface
- */
-
+/**
+ * gnome_font_face_get_num_glyphs:
+ * @face: 
+ * 
+ * Returns the number of glyphs in the font
+ * 
+ * Return Value: number of glyphs, 0 on error
+ **/
 gint
 gnome_font_face_get_num_glyphs (GnomeFontFace * face)
 {
@@ -313,7 +352,7 @@ gnome_font_face_get_num_glyphs (GnomeFontFace * face)
 	g_return_val_if_fail (GNOME_IS_FONT_FACE (face), 0);
 
 	if (!GFF_LOADED (face)) {
-		g_warning ("file %s: line %d: Face %s: Cannot load face", __FILE__, __LINE__, face->entry->name);
+		g_warning ("Could not load FACE %s, inside _get_num_glyphs", face->entry->name);
 		return 0;
 	}
 
@@ -442,10 +481,9 @@ gnome_font_face_get_glyph_stdkerning (GnomeFontFace *face, gint glyph0, gint gly
 }
 
 /*
- * Give (possibly localized) demonstration text for given face
+ * Give (possibly localized) demonstration text for a given face
  * Most usually this tells about quick fox and lazy dog...
  */
-
 const guchar *
 gnome_font_face_get_sample (GnomeFontFace * face)
 {
@@ -455,7 +493,6 @@ gnome_font_face_get_sample (GnomeFontFace * face)
 /*
  * Creates new face and creates link with FontEntry
  */
-
 static void
 gff_face_from_entry (GPFontEntry *e)
 {
@@ -515,33 +552,6 @@ gnome_font_face_find_closest (const guchar *name)
 	if (name)
 		face = gnome_font_face_find (name);
 
-	if (!face) {
-		/* No match, try default */
-		guchar *locale, *p;
-		guchar c[128];
-		GPFontEntry *e;
-		locale = g_strdup (setlocale (LC_ALL, NULL));
-		if (!locale)
-			locale = g_strdup ("C");
-		strncpy (c, locale, 127);
-		c[127] = '\0';
-		p = c;
-		while (isalpha (*p) || (*p == '_')) p += 1;
-		if (*p)
-			*p = '\0';
-		e = g_hash_table_lookup (map->defaultsdict, c);
-		if (!e)
-			e = g_hash_table_lookup (map->defaultsdict, "C");
-		if (e) {
-			if (e->face) {
-				gnome_font_face_ref (e->face);
-			} else {
-				gff_face_from_entry (e);
-			}
-			face = e->face;
-		}
-		g_free (locale);
-	}
 	if (!face && map->fonts) {
 		/* No face, no default, load whatever font is first */
 		GPFontEntry *e;
@@ -763,6 +773,7 @@ gff_load (GnomeFontFace *face)
 	FT_CharMap charmap;
 	FT_CharMap appleRoman = 0;
 	FT_CharMap symbol = 0;
+	const guchar *psname;
 	int n;
 
 	if (!ft_library) {
@@ -776,41 +787,19 @@ gff_load (GnomeFontFace *face)
 		entry = face->entry;
 	}
 
-	if (GFE_IS_T1_A (entry)) {
-		GPFontEntryT1 *t1;
-		t1 = (GPFontEntryT1 *) entry;
-		ft_result = FT_New_Face (ft_library, t1->pfb.name, 0, &ft_face);
-		g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
-		if (t1->afm.name) {
-			ft_result = FT_Attach_File (ft_face, t1->afm.name);
-			if (ft_result != FT_Err_Ok) {
-				g_warning ("file %s: line %d: face %s: Cannot attach file %s", __FILE__, __LINE__, face->entry->name, t1->afm.name);
-			}
-		}
-	} else if (GFE_IS_TT (entry)) {
-		GPFontEntryTT *tt;
-		tt = (GPFontEntryTT *) entry;
-		ft_result = FT_New_Face (ft_library, tt->ttf.name, tt->facenum, &ft_face);
-		g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
-	} else if (GFE_IS_S (entry)) {
-		GPFontEntrySpecial *s;
-		GSList *l;
-		s = (GPFontEntrySpecial *) entry;
-		ft_result = FT_New_Face (ft_library, s->file.name, 0, &ft_face);
-		g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
-		for (l = s->additional; l != NULL; l = l->next) {
-			ft_result = FT_Attach_File (ft_face, (gchar *) l->data);
-			if (ft_result != FT_Err_Ok) {
-				g_warning ("file %s: line %d: face %s: Cannot attach file %s",
-					   __FILE__, __LINE__, face->entry->name, (gchar *) l->data);
-			}
-		}
-	} else {
-		g_assert_not_reached ();
-	}
+	ft_result = FT_New_Face (ft_library, entry->file, 0, &ft_face);
+	g_return_val_if_fail (ft_result == FT_Err_Ok, FALSE);
 
-	/* fixme: scalability */
-	/* fixme: glyph names */
+	psname = FT_Get_Postscript_Name (ft_face);
+	if (psname == NULL) {
+		g_warning ("PS name is NULL, for \"%s\" using fallback", entry->file);
+		face->psname = g_strdup ("Helvetica");
+	} else {
+		face->psname = g_strdup (psname);
+	}
+	
+	/* FIXME: scalability (Lauris) */
+	/* FIXME: glyph names (Lauris) */
 
 	face->ft_face = ft_face;
 
@@ -1067,7 +1056,7 @@ gnome_font_face_pso_new (GnomeFontFace *face, guchar *residentname)
 	g_object_ref (G_OBJECT (face));
 	if (residentname)
 		pso->residentname = g_strdup (residentname);
-	pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->entry->psname);
+	pso->encodedname = g_strdup_printf ("GnomeUni-%s", face->psname);
 	pso->bufsize = 0;
 	pso->length = 0;
 	pso->buf = NULL;
@@ -1127,7 +1116,6 @@ gnome_font_face_ps_embed (GnomeFontPsObject *pso)
 
 	switch (pso->face->entry->type) {
 	case GP_FONT_ENTRY_TYPE1:
-	case GP_FONT_ENTRY_TYPE1_ALIAS:
 		gnome_font_face_ps_embed_t1 (pso);
 		break;
 	case GP_FONT_ENTRY_TRUETYPE:
@@ -1146,19 +1134,14 @@ gnome_font_face_ps_embed (GnomeFontPsObject *pso)
 static void
 gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 {
-	GPFontEntryT1 *t1;
-	struct stat st;
+	GnomePrintBuffer b;
 	guchar * fbuf;
-	gint fh;
+	const gchar *file_name;
 	const gchar *embeddedname;
 
-	t1 = (GPFontEntryT1 *) pso->face->entry;
-
-	if (pso->face->entry->type == GP_FONT_ENTRY_TYPE1) {
-		embeddedname = pso->face->entry->psname;
-	} else {
-		embeddedname = ((GPFontEntryT1Alias *) pso->face->entry)->alias;
-	}
+	g_return_if_fail (pso->face->entry->type == GP_FONT_ENTRY_TYPE1);
+	file_name = pso->face->entry->file;	
+	embeddedname = pso->face->psname;
 
 	if (!GFF_LOADED (pso->face)) {
 		g_warning ("file %s: line %d: Face: %s: Cannot load face", __FILE__, __LINE__, pso->face->entry->name);
@@ -1166,26 +1149,13 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 		return;
 	}
 
-	/* Step 0: mmap pfb file */
-	fh = open (t1->pfb.name, O_RDONLY);
-	if (fh < 0) {
-		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, t1->pfb.name);
-		gnome_font_face_ps_embed_empty (pso);
-		return;
-	}
-	if (fstat (fh, &st) != 0) {
-		g_warning ("file %s: line %d: Cannot fstat font file %s", __FILE__, __LINE__, t1->pfb.name);
-		gnome_font_face_ps_embed_empty (pso);
-		return;
-	}
-	fbuf = mmap (NULL, st.st_size, PROT_READ, MAP_SHARED, fh, 0);
-	close (fh);
-	if ((fbuf == NULL) || (fbuf == (guchar *) -1)) {
-		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, t1->pfb.name);
+	if (GNOME_PRINT_OK != gnome_print_buffer_mmap (&b, file_name)) {
+		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, file_name);
 		gnome_font_face_ps_embed_empty (pso);
 		return;
 	}
 
+	fbuf = b.buf;
 	/* Step 1: */	   
 	if (*fbuf == 0x80) {
 		const char hextab[16] = "0123456789abcdef";
@@ -1195,10 +1165,10 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 		 * Language Fonts", page 9
 		 */
 		idx = 0;
-		while (idx < st.st_size) {
+		while (idx < b.buf_size) {
 			gint length, i;
 			if (fbuf[idx] != 0x80) {
-				g_warning ("file %s: line %d: Corrupt %s", __FILE__, __LINE__, t1->pfb.name);
+				g_warning ("file %s: line %d: Corrupt %s", __FILE__, __LINE__, file_name);
 				gnome_font_face_ps_embed_empty (pso);
 				return;
 			}
@@ -1227,23 +1197,22 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 				/* Finished */
 				gnome_font_face_ps_embed_ensure_size (pso, 1);
 				pso->buf[pso->length++] = '\n';
-				idx = st.st_size;
+				idx = b.buf_size;
 				break;
 			default:
-				g_warning ("file %s: line %d: Corrupt %s", __FILE__, __LINE__, t1->pfb.name);
+				g_warning ("file %s: line %d: Corrupt %s", __FILE__, __LINE__, file_name);
 				gnome_font_face_ps_embed_empty (pso);
 				return;
 			}
 		}
 	} else {
-		gnome_font_face_ps_embed_ensure_size (pso, st.st_size);
-		memcpy (pso->buf, fbuf, st.st_size);
-		pso->buf[st.st_size] = '\0';
-		pso->length = st.st_size;
+		gnome_font_face_ps_embed_ensure_size (pso, b.buf_size + 1);
+		memcpy (pso->buf, fbuf, b.buf_size);
+		pso->buf[b.buf_size] = '\0';
+		pso->length = b.buf_size;
 	}
 
-	/* Unmap file */
-	munmap (fbuf, st.st_size);
+	gnome_print_buffer_munmap (&b);
 
 	if (pso->encodedbytes == 1) {
 		gint glyph;
@@ -1258,7 +1227,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 			g = (glyph < pso->face->num_glyphs) ? glyph : 0;
 			status = FT_Get_Glyph_Name (pso->face->ft_face, g, c, 256);
 			if (status != FT_Err_Ok) {
-				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, g, t1->pfb.name);
+				g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, g, file_name);
 				g_snprintf (c, 256, ".notdef");
 			}
 			gf_pso_sprintf (pso, ((glyph & 0xf) == 0xf) ? "/%s\n" : "/%s ", c);
@@ -1296,7 +1265,7 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
 					glyph = 0;
 				status = FT_Get_Glyph_Name (pso->face->ft_face, glyph, c, 256);
 				if (status != FT_Err_Ok) {
-					g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, glyph, t1->pfb.name);
+					g_warning ("file %s: line %d: Glyph %d has no name in %s", __FILE__, __LINE__, glyph, file_name);
 					g_snprintf (c, 256, ".notdef");
 				}
 				gf_pso_sprintf (pso, ((j & 0xf) == 0xf) ? "/%s\n" : "/%s ", c);
@@ -1323,44 +1292,34 @@ gnome_font_face_ps_embed_t1 (GnomeFontPsObject *pso)
  * Adobe Inc., The Type42 Font Format Specification, 1998 <http://partners.adobe.com/asn/developer/PDFS/TN/5012.Type42_Spec.pdf>
  */
 
+#define TT_BLOCK_SIZE 1024
+
 static void
 gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 {
-	GPFontEntryTT *tt;
-	struct stat st;
+	GnomePrintBuffer b;
+	const gchar *file_name;
 	guchar *fbuf;
-	gint fh;
 	GSList *strings;
 	const gchar * embeddedname;
 	gdouble TTVersion, MfrRevision;
 	const ArtDRect *bbox;
 	gint i;
+	gchar *line;
+	gint line_length;
 
-	tt = (GPFontEntryTT *) pso->face->entry;
+	g_return_if_fail (pso->face->entry->type == GP_FONT_ENTRY_TRUETYPE);
+ 	file_name = pso->face->entry->file;
 
-	/* mmap file */
-	fh = open (tt->ttf.name, O_RDONLY);
-	if (fh < 0) {
-		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, tt->ttf.name);
-		gnome_font_face_ps_embed_empty (pso);
-		return;
-	}
-	if (fstat (fh, &st) != 0) {
-		g_warning ("file %s: line %d: Face: %s: Cannot fstat face", __FILE__, __LINE__, tt->ttf.name);
-		gnome_font_face_ps_embed_empty (pso);
-		return;
-	}
-	fbuf = mmap (NULL, st.st_size, PROT_READ, MAP_SHARED, fh, 0);
-	close (fh);
-	if ((fbuf == NULL) || (fbuf == (guchar *) -1)) {
-		g_warning ("file %s: line %d: Cannot open font file %s", __FILE__, __LINE__, tt->ttf.name);
+	if (GNOME_PRINT_OK != gnome_print_buffer_mmap (&b, file_name)) {
 		gnome_font_face_ps_embed_empty (pso);
 		return;
 	}
+	
+	embeddedname = pso->face->psname;
 
-	embeddedname = pso->face->entry->psname;
-
-	strings = gp_tt_split_file (fbuf, st.st_size);
+	fbuf = b.buf;
+	strings = gp_tt_split_file (fbuf, b.buf_size);
 	if (strings) {
 		/* Relatively easy - just embed TTF into PS stream */
 		/* Download master font */
@@ -1381,23 +1340,27 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		/* fixme: XUID */
 		/* fixme: Be more intelligent */
 		gf_pso_sprintf (pso, "/sfnts [\n");
+		line_length = gnome_print_encode_hex_wcs (TT_BLOCK_SIZE);
+		line = g_malloc (line_length);
 		while (strings) {
-			guint start, next, size, i;
+			guint start, next;
+			guchar *s, *e;
 			start = GPOINTER_TO_UINT (strings->data);
 			strings = g_slist_remove (strings, strings->data);
-			next = strings ? GPOINTER_TO_UINT (strings->data) : st.st_size;
-			size = next - start;
+			next = strings ? GPOINTER_TO_UINT (strings->data) : b.buf_size;
 			gf_pso_sprintf (pso, "<\n");
-			for (i = start; i < next; i+= 32) {
-				gint e, j;
-				e = MIN (i + 32, next);
-				for (j = i; j < e; j++) {
-					gf_pso_sprintf (pso, "%.2x", *(fbuf + j));
-				}
-				gf_pso_sprintf (pso, "\n");
+			s = b.buf + start;
+			e = b.buf + next;
+			while (s < e) {
+				gint block_size, len;
+				block_size = MIN (TT_BLOCK_SIZE, e - s);
+				len = gnome_print_encode_hex (s, line, block_size);
+				gf_pso_print_sized (pso, line, len);
+				s += block_size;
 			}
 			gf_pso_sprintf (pso, strings ? ">\n" : "00>\n");
 		}
+		g_free (line);
 		gf_pso_sprintf (pso, "] def\n");
 		/* fixme: Use CID or something */
 		gf_pso_sprintf (pso, "/CharStrings %d dict dup begin\n", pso->face->num_glyphs);
@@ -1412,8 +1375,8 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		/* This is somewhat experimental - convert TTF to Type1 */
 		afm = ttf2pfa (pso->face->ft_face, embeddedname, pso->glyphs);
 		if (!afm) {
-			munmap (fbuf, st.st_size);
-			g_warning ("file %s: line %d: Cannot convert TTF %s to Type1", __FILE__, __LINE__, tt->ttf.name);
+			gnome_print_buffer_munmap (&b);
+			g_warning ("file %s: line %d: Cannot convert TTF %s to Type1", __FILE__, __LINE__, file_name);
 			gnome_font_face_ps_embed_empty (pso);
 			return;
 		}
@@ -1423,8 +1386,7 @@ gnome_font_face_ps_embed_tt (GnomeFontPsObject *pso)
 		pso->length = pso->bufsize;
 	}
 
-	/* Unmap font file */
-	munmap (fbuf, st.st_size);
+	gnome_print_buffer_munmap (&b);
 
 	/*
 	 * We have font downloaded (hopefully)
@@ -1541,7 +1503,15 @@ gnome_font_face_ps_embed_empty (GnomeFontPsObject *pso)
 }
 
 static void
-gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...)
+gf_pso_print_sized (GnomeFontPsObject *pso, const guchar *text, gint size)
+{
+	gnome_font_face_ps_embed_ensure_size (pso, size);
+	memcpy (pso->buf + pso->length, text, size);
+	pso->length += size;
+}
+
+static void
+gf_pso_sprintf (GnomeFontPsObject *pso, const gchar * format, ...)
 {
 	va_list arguments;
 	gchar *oldlocale;
@@ -1556,9 +1526,7 @@ gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...)
 	va_end (arguments);
 
 	len = strlen (text);
-	gnome_font_face_ps_embed_ensure_size (pso, len);
-	memcpy (pso->buf + pso->length, text, len);
-	pso->length += len;
+	gf_pso_print_sized (pso, text, len);
 	g_free (text);
 
 	setlocale (LC_NUMERIC, oldlocale);
@@ -1568,12 +1536,14 @@ gf_pso_sprintf (GnomeFontPsObject * pso, const gchar * format, ...)
 static void
 gnome_font_face_ps_embed_ensure_size (GnomeFontPsObject *pso, gint size)
 {
-	if (pso->length + size > pso->bufsize) {
+	gint need = pso->length + size;
+
+	if (need > pso->bufsize) {
 		if (pso->bufsize < 1) {
 			pso->bufsize = MAX (size, 1024);
 			pso->buf = g_new (guchar, pso->bufsize);
 		} else {
-			while (pso->length + size > pso->bufsize)
+			while (need > pso->bufsize)
 				pso->bufsize <<= 1;
 			pso->buf = g_renew (guchar, pso->buf, pso->bufsize);
 		}
