@@ -912,7 +912,7 @@ gp_ctx_new (void)
 	ctx->dash_flag = GP_GC_FLAG_UNSET;
 	ctx->privatedash = FALSE;
 
-	ctx->font = gnome_font_find_closest ("Helvetica", 12.0);
+	ctx->font = gnome_font_find_closest ("Sans Regular", 12.0);
 	ctx->font_flag = GP_GC_FLAG_UNSET;
 
 	g_return_val_if_fail (ctx->font != NULL, NULL);
