@@ -112,6 +112,9 @@ gpa_settings_finalize (GObject *object)
 	settings->printer = NULL;
 	settings->model   = NULL;
 
+	g_free (settings->name);
+	settings->name = NULL;
+
 	child = GPA_NODE (settings)->children;
 	while (child) {
 		GPANode *next;
